from knack.help_files import helps

helps[
    "image"
] = """
type: group
short-summary: Operations related to container images
"""

helps[
    "fs"
] = """
type: group
short-summary: Operations related to file system
"""

helps[
    "git"
] = """
type: group
short-summary: Operations related to Git repository
"""

helps[
    "rules"
] = """
type: group
short-summary: Maintain rule sets
"""

helps[
    "vulndb"
] = """
type: group
short-summary: Vulnerability database tasks
"""
